@@ -1,5 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*- */
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
+ *
  * gtksourceprintcompositor.c
  * This file is part of GtkSourceView 
  *
@@ -34,25 +34,6 @@
 #include "gtksourceview-i18n.h" 
 #include "gtksourceprintcompositor.h"
 
-/**
- * SECTION:printcompositor
- * @Short_description: Compose a #GtkSourceBuffer for printing
- * @Title: GtkSourcePrintCompositor
- *
- * The #GtkSourcePrintCompositor object is used to compose a #GtkSourceBuffer
- * for printing. You can set various configuration options to customize the
- * printed output. #GtkSourcePrintCompositor is designed to be used with the
- * high-level printing API of gtk+, i.e. #GtkPrintOperation.
- *
- * The margins specified in this object are the layout margins: they define the
- * blank space bordering the printed area of the pages. They must not be
- * confused with the "print margins", i.e. the parts of the page that the
- * printer cannot print on, defined in the #GtkPageSetup objects. If the
- * specified layout margins are smaller than the "print margins", the latter
- * ones are used as a fallback by the #GtkSourcePrintCompositor object, so that
- * the printed area is not clipped.
- */
-
 /*
 #define ENABLE_DEBUG
 #define ENABLE_PROFILE
@@ -316,7 +297,7 @@ gtk_source_print_compositor_set_property (GObject      *object,
 	switch (prop_id)
 	{
 		case PROP_BUFFER:
-			compositor->priv->buffer = GTK_SOURCE_BUFFER (g_value_dup_object (value));
+			compositor->priv->buffer = GTK_SOURCE_BUFFER (g_value_get_object (value));
 			break;
 		case PROP_TAB_WIDTH:
 			gtk_source_print_compositor_set_tab_width (compositor,
@@ -395,7 +376,7 @@ gtk_source_print_compositor_finalize (GObject *object)
 
 	if (compositor->priv->footer_font != NULL)
 		pango_font_description_free (compositor->priv->footer_font);
-
+		
 	g_free (compositor->priv->header_format_left);
 	g_free (compositor->priv->header_format_right);
 	g_free (compositor->priv->header_format_center);
@@ -406,21 +387,6 @@ gtk_source_print_compositor_finalize (GObject *object)
 	G_OBJECT_CLASS (gtk_source_print_compositor_parent_class)->finalize (object);
 }
 
-static void
-gtk_source_print_compositor_dispose (GObject *object)
-{
-	GtkSourcePrintCompositor *compositor;
-
-	compositor = GTK_SOURCE_PRINT_COMPOSITOR (object);
-
-	if (compositor->priv->buffer != NULL) {
-		g_object_unref (compositor->priv->buffer);
-		compositor->priv->buffer = NULL;
-	}
-
-	G_OBJECT_CLASS (gtk_source_print_compositor_parent_class)->dispose (object);
-}
-
 static void						 
 gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 {
@@ -431,7 +397,6 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	object_class->get_property = gtk_source_print_compositor_get_property;
 	object_class->set_property = gtk_source_print_compositor_set_property;
 	object_class->finalize = gtk_source_print_compositor_finalize;
-	object_class->dispose = gtk_source_print_compositor_dispose;
 
 	/**
 	 * GtkSourcePrintCompositor:buffer:
@@ -789,8 +754,6 @@ gtk_source_print_compositor_new (GtkSourceBuffer *buffer)
  * #GtkSourcePrintCompositor:wrap-mode, #GtkSourcePrintCompositor:body-font-name and
  * #GtkSourcePrintCompositor:print-line-numbers.
  *
- * Return value: a new print compositor object.
- *
  * Since: 2.2
  **/
 GtkSourcePrintCompositor *
@@ -2261,19 +2224,11 @@ calculate_page_size_and_margins (GtkSourcePrintCompositor *compositor,
 	});						 
 }
 
-/* TODO: maybe we should have a public api to set
- * which tags need to be printed and which should not.
- * For now we special case bracket matches.
- */
 static gboolean
 ignore_tag (GtkSourcePrintCompositor *compositor,
             GtkTextTag               *tag)
 {
-	GtkTextTag *bm_tag;
-
-	bm_tag = _gtk_source_buffer_get_bracket_match_tag (compositor->priv->buffer);
-	if ((bm_tag != NULL) && (tag == bm_tag))
-		return TRUE;
+	/* TODO: ignore bracket match tags etc */
 
 	return FALSE;
 }
@@ -2691,7 +2646,7 @@ gtk_source_print_compositor_paginate (GtkSourcePrintCompositor *compositor,
 				PangoRectangle logical_rect;
 				gboolean is_first_line = TRUE;
 				double part_height = 0;
-				gint idx;
+				gint index;
 
 				layout_iter = pango_layout_get_iter (compositor->priv->layout);
 
@@ -2723,9 +2678,9 @@ gtk_source_print_compositor_paginate (GtkSourcePrintCompositor *compositor,
 				 * may start in the middle of a line, so we have
 				 * to add.
 				 */
-				idx = gtk_text_iter_get_line_index (&start);
-				idx += pango_layout_iter_get_index (layout_iter);
-				gtk_text_iter_set_line_index (&start, idx);
+				index = gtk_text_iter_get_line_index (&start);
+				index += pango_layout_iter_get_index (layout_iter);
+				gtk_text_iter_set_line_index (&start, index);
 
 				pango_layout_iter_free (layout_iter);
 
@@ -3133,7 +3088,6 @@ gtk_source_print_compositor_draw_page (GtkSourcePrintCompositor *compositor,
 
 	g_return_if_fail (GTK_IS_SOURCE_PRINT_COMPOSITOR (compositor));
 	g_return_if_fail (GTK_IS_PRINT_CONTEXT (context));
-	g_return_if_fail (page_nr >= 0);
 
 	compositor->priv->current_page = page_nr;
 
@@ -3206,13 +3160,13 @@ gtk_source_print_compositor_draw_page (GtkSourcePrintCompositor *compositor,
 
 	g_return_if_fail (compositor->priv->buffer != NULL);
 	g_return_if_fail (compositor->priv->pages != NULL);
-	g_return_if_fail ((guint) page_nr < compositor->priv->pages->len);
+	g_return_if_fail (page_nr < compositor->priv->pages->len);
 
 	offset = g_array_index (compositor->priv->pages, int, page_nr);
 	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (compositor->priv->buffer),
 					    &start, offset);
 
-	if ((guint) page_nr + 1 < compositor->priv->pages->len)
+	if (page_nr + 1 < compositor->priv->pages->len)
 	{
 		offset = g_array_index (compositor->priv->pages, int, page_nr + 1);
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (compositor->priv->buffer),
